import asyncio
import websockets
import argparse
import runpy
import json
from http import HTTPStatus
import traceback
import sys, os
from datetime import datetime

currentdir = os.path.dirname(os.path.realpath(__file__))
parentdir = os.path.dirname(currentdir)
sys.path.append(parentdir)

from intelli_utilities import BaiException

parser = argparse.ArgumentParser()
parser.add_argument('--port', help='websocket port to use')
args = parser.parse_args()

import logging
logging.basicConfig(
    filename='log/socket.log',
    encoding='utf-8',
    level=logging.ERROR,
    format='%(asctime)s - %(levelname)s - %(message)s'
)

def run_script(req_data):
    args = req_data['payload']
    """
    All set variable in 'scriptFile' will be avaiable in data variable as dict key.
    Which means 'scriptFile' must set a variable named response for return data.
    Set a variable named 'error' to return error statements
    """
    data = runpy.run_path(path_name='filedb/scripts/{}/{}'.format(req_data['scriptType'], req_data['scriptFile']), run_name='__main__', init_globals={'runpy_args': args})
    return data['response']

async def handle_connection(websocket):
    response = {
        'topic': '',
        'data': None,
        'statusCode': HTTPStatus.OK,
        'error': None
    }
    try:        
        async for message in websocket:
            try:
                req_data = json.loads(message)
                response['topic'] = req_data['payload']['topic']
                response['data'] = run_script(req_data)
                response['statusCode'] = HTTPStatus.OK
                response['error'] = None
            except BaiException as be:
                response['statusCode'] = HTTPStatus.INTERNAL_SERVER_ERROR
                now = datetime.now()
                error = "{}: {}".format(now.strftime('%Y-%m-%dT%H:%M:%S%z'), be)
                response['error'] = error
            except Exception as e:
                response['statusCode'] = HTTPStatus.INTERNAL_SERVER_ERROR
                exc_type, _, exc_tb = sys.exc_info()
                fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]        
                now = datetime.now()
                error = "{}: {} {} {} {} {}".format(now.strftime('%Y-%m-%dT%H:%M:%S%z'), e, exc_type, fname, exc_tb.tb_lineno, traceback.format_exc())
                response['error'] = error
                logging.error(error)

            await websocket.send(json.dumps(response))

    except websockets.exceptions.ConnectionClosed:
        logging.info("Connection closed normally")
    except Exception as e:
        logging.error(f"WebSocket error: {str(e)}")

async def start_server():
    HOST = '127.0.0.1'
    PORT = int(args.port)
    
    async with websockets.serve(handle_connection, HOST, PORT):
        print("SERVER_READY", flush=True)
        await asyncio.Future()  # run forever

if __name__ == "__main__":
    asyncio.run(start_server())